require 'ams_Lib'

# AMS is a top level namespace of AMS Library. AMS stands for Anton M Synytsia.
# @since 2.0.0
module AMS
  class << self

    # Validate object type.
    # @param [Object] object
    # @param [Object, Array<Object>] types An object or an array of objects to
    #   check against.
    # @return [void]
    # @raise [TypeError] if object class doesn't match with any of the specified
    #   types.
    def validate_type(object, *types)
      types = types.flatten
      return if types.empty?
      types.each { |type| return if object.is_a?(type) }
      string = case types.size
      when 1
        types[0].to_s
      when 2
        "#{types[0]} or #{types[1]}"
      else
        "#{types[0...-1].join(', ')}, or #{types[-1]}"
      end
      if RUBY_VERSION =~ /1.8/
        ::Kernel.raise(TypeError, "Expected #{string}, but got #{object.class.to_s}.", caller)
      else
        ::Kernel.raise(TypeError, "Expected #{string.encode(Encoding::UTF_8)}, but got #{object.class.to_s.encode(Encoding::UTF_8)}.", caller)
      end
    end

    # Clamp value between min and max.
    # @param [Numeric] value
    # @param [Numeric, nil] min Pass +nil+ to have no min limit.
    # @param [Numeric, nil] max Pass +nil+ to have no max limit.
    # @return [Numeric]
    def clamp(value, min, max)
      if min and value < min
        min
      elsif max and value > max
        max
      else
        value
      end
    end

    # Get numeric value sign.
    # @param [Numeric] value
    # @return [Fixnum] -1, 0, or 1
    def sign(value)
      value.zero? ? 0 : (value > 0 ? 1 : -1)
    end

    # Get least value of the two values.
    # @param [Numeric] a
    # @param [Numeric] b
    # @return [Numeric]
    def min(a, b)
      a < b ? a : b
    end

    # Get greatest value of the two values.
    # @param [Numeric] a
    # @param [Numeric] b
    # @return [Numeric]
    def max(a, b)
      a > b ? a : b
    end

    # Scale vector.
    # @param [Array<Numeric>, Geom::Vector3d] vector
    # @param [Numeric] scale
    # @return [Geom::Vector3d]
    def scale_vector(vector, scale)
      Geom::Vector3d.new(vector[0]*scale, vector[1]*scale, vector[2]*scale)
    end

    # Get entity by entity ID.
    # @param [Fixnum] id
    # @return [Sketchup::Entity, nil]
    def get_entity_by_id(id)
      model = ::Sketchup.active_model
      model.entities.each { |e| return e if e.entityID == id }
      model.definitions.each { |d|
        return d if d.entityID == id
        d.instances.each { |e| return e if e.entityID == id }
        d.entities.each { |e| return e if e.entityID == id }
      }
      model.materials.each { |e| return e if e.entityID == id }
      model.layers.each { |e| return e if e.entityID == id }
      model.styles.each { |e| return e if e.entityID == id }
      model.pages.each { |e| return e if e.entityID == id }
      nil
    end

    # Get top level entity by entity ID.
    # @param [Fixnum] id
    # @return [Sketchup::Entity, nil]
    # @since 3.0.0
    def get_top_entity_by_id(id)
      ::Sketchup.active_model.entities.each { |e|
        return e if e.entityID == id
      }
      nil
    end

    # Determine whether object is +true+ or +false+.
    # @return [Boolean]
    def is_boolean?(object)
      object.is_a?(TrueClass) || object.is_a?(FalseClass)
    end

    alias is_bool? is_boolean?

  end # class << self
end # module AMS


# The Lib namespace contains a set of utility functions that make it easy for
# the user to control particular files within the library.
# @since 2.0.0
module AMS::Lib

  # Folder location
  PATH = RUBY_VERSION =~ /1.8/ ? ::File.dirname(__FILE__).freeze : ::File.dirname(__FILE__).encode(Encoding::UTF_8).freeze
  # Name of the main file
  BASE = RUBY_VERSION =~ /1.8/ ? ::File.basename(__FILE__).freeze : ::File.basename(__FILE__).encode(Encoding::UTF_8).freeze

  # @!visibility private
  REGISTERED_FILES = [
    'main.rb',
    'multi_line_text.rb',
    'ray_util.rb',
    'system.rb',
    'translate.rb'
  ].freeze

  class << self

    # Require all library.
    # @note All already loaded files will be ignored.
    # @return [Fixnum] The number of files loaded.
    def require_all
      count = 0
      ::Dir.glob( ::File.join(PATH, '*.{rb, rbs}') ).each { |fpath|
        fpath2 = RUBY_VERSION =~ /1.8/ ? fpath : fpath.encode(Encoding::UTF_8)
        base_name = ::File.basename(fpath2)
        next if base_name == BASE || !REGISTERED_FILES.include?(base_name)
        res = ::Kernel.require(fpath2)
        count += 1 if res
      }
      count
    end

    # Load all library.
    # @note All already loaded files will be reloaded.
    # @return [Fixnum] The number of files loaded.
    def load_all
      count = 0
      ::Dir.glob( ::File.join(PATH, '*.{rb, rbs}') ).each { |fpath|
        fpath2 = RUBY_VERSION =~ /1.8/ ? fpath : fpath.encode(Encoding::UTF_8)
        base_name = ::File.basename(fpath2)
        next if base_name == BASE || !REGISTERED_FILES.include?(base_name)
        ::Kernel.load(fpath2)
        count += 1
      }
      count
    end

    # Require specific file within the library's path.
    # @note The file will not be loaded if it was already loaded.
    # @param [String] file_name
    # @return [Boolean] success
    def require(file_name)
      fpath = RUBY_VERSION =~ /1.8/ ? ::File.join(PATH, file_name) : ::File.join(PATH, file_name.encode(Encoding::UTF_8))
      ::Kernel.require(fpath)
    end

    # Load specific file within the library's path.
    # @note The file will be reloaded if it was already loaded.
    # @param [String] file_name
    # @return [Boolean] success
    def load(file_name)
      fpath = RUBY_VERSION =~ /1.8/ ? ::File.join(PATH, file_name) : ::File.join(PATH, file_name.encode(Encoding::UTF_8))
      ::Kernel.load(fpath)
    end

    # Remove unregistered Ruby files from the library.
    # @return [Fixnum] Number of files deleted.
    # @since 2.1.0
    def clean_up
      count = 0
      ::Dir.glob( ::File.join(PATH, '*.{rb, rbs}') ).each { |fpath|
        fpath2 = RUBY_VERSION =~ /1.8/ ? fpath : fpath.encode(Encoding::UTF_8)
        base_name = ::File.basename(fpath2)
        next if base_name == BASE || REGISTERED_FILES.include?(base_name)
        ::File.delete(fpath2)
        count += 1
      }
      count
    end

  end # class << self
end # module AMS::Lib

unless file_loaded?(__FILE__)
  file_loaded(__FILE__)

  ops = (RUBY_PLATFORM =~ /mswin|mingw/i) ? 'win' : 'mac'
  bit = (Sketchup.respond_to?('is_64bit?') && Sketchup.is_64bit? )? '64' : '32'
  ver = (RUBY_VERSION =~ /1.8/) ? '1.8' : '2.0'
  ext = (RUBY_PLATFORM =~ /mswin|mingw/i) ? '.so' : '.a'

  begin
    require File.join(AMS::Lib::PATH, ops+bit, ver, 'ams_lib' + ext)
  rescue LoadError => e
    # Avoid loading the library if it isn't compatible with the current Ruby build.
    msg = e.message
    trace = e.backtrace.join("\n")
    if RUBY_VERSION !~ /1.8/
      msg.encode!(Encoding::UTF_8)
      trace.encode!(Encoding::UTF_8)
    end
    UI.messagebox("An error occurred while loading ams_lib#{ext}! This error might be caused due to AMS Library not being compatible on your system.\n\nError:\n#{msg}\n\nLocation Trace:\n#{trace}")
    raise e
  end

  begin
    require File.join(AMS::Lib::PATH, ops+bit, ver, 'ams_win32_api' + ext)
  rescue LoadError => e
    # Avoid loading the library if it isn't compatible with the current Ruby build.
    msg = e.message
    trace = e.backtrace.join("\n")
    if RUBY_VERSION !~ /1.8/
      msg.encode!(Encoding::UTF_8)
      trace.encode!(Encoding::UTF_8)
    end
    UI.messagebox("An error occurred while loading ams_win32_api#{ext}! This error might be caused due to AMS Library not being compatible on your system.\n\nError:\n#{msg}\n\nLocation Trace:\n#{trace}")
    raise e
  end

  #~ AMS::Lib.clean_up
  AMS::Lib.require_all
end
